#define G_LOG_DOMAIN "ap-sheet"

#include "ap-sheet.h"

#include <glib.h>
#include <glib-object.h>

typedef struct _APSheetPrivate
{
    gchar *filename;

    GHashTable *characteristics;
    GHashTable *attributes;
    GHashTable *abilities;
} APSheetPrivate;

struct _APSheet
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APSheet, ap_sheet, G_TYPE_OBJECT)

enum
{
    PROP_0,
    PROP_FILE,
    N_PROPERTIES
};

static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };

void
ap_sheet_add_characteristic (APSheet *self,
			                 gchar   *key,
                             gchar   *label,
			                 gchar   *value)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);
    APSheetCharacteristic *characteristic;

    g_return_if_fail (AP_IS_SHEET (self));
    g_return_if_fail (key != NULL);
    g_return_if_fail (label != NULL);

    characteristic = g_new (APSheetCharacteristic, 1);
    characteristic->label = label;
    characteristic->value = value;

    g_hash_table_insert (priv->characteristics, key, characteristic);
}

APSheetCharacteristic *
ap_sheet_get_characteristic (APSheet *self,
                             gchar   *key)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);

    g_return_val_if_fail (AP_IS_SHEET (self), NULL);
    g_return_val_if_fail (key != NULL, NULL);

    return g_hash_table_lookup (priv->characteristics, key);
}

gchar *
ap_sheet_get_characteristic_label (APSheet *self,
                                   gchar   *key)
{
    APSheetCharacteristic *characteristic;

    g_return_val_if_fail (AP_IS_SHEET (self), NULL);
    g_return_val_if_fail (key != NULL, NULL);

    characteristic = ap_sheet_get_characteristic (self, key);

    if (characteristic != NULL)
        return characteristic->label;
    else
        return NULL;
}

gchar *
ap_sheet_get_characteristic_value (APSheet *self,
                                   gchar   *key)
{
    APSheetCharacteristic *characteristic;

    g_return_val_if_fail (AP_IS_SHEET (self), NULL);
    g_return_val_if_fail (key != NULL, NULL);

    characteristic = ap_sheet_get_characteristic (self, key);

    if (characteristic != NULL)
        return characteristic->value;
    else
        return NULL;
}


void
ap_sheet_add_attribute (APSheet *self,
	                    gchar   *key,
                        gchar   *label,
	                    guint    value)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);
    APSheetAttribute *attribute;

    g_return_if_fail (AP_IS_SHEET (self));
    g_return_if_fail (key != NULL);
    g_return_if_fail (label != NULL);

    attribute = g_new (APSheetAttribute, 1);
    attribute->label = label;
    attribute->value = value;

    g_hash_table_insert (priv->attributes, key, attribute);
}

void
ap_sheet_add_ability (APSheet *self,
                      gchar   *key,
                      gchar   *label,
                      gint     value)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);
    APSheetAbility *ability;

    g_return_if_fail (AP_IS_SHEET (self));
    g_return_if_fail (key != NULL);
    g_return_if_fail (label != NULL);

    ability = g_new (APSheetAbility, 1);
    ability->label = label;
    ability->value = value;

    g_hash_table_insert (priv->abilities, key, ability);
}









static void
ap_sheet_set_property (GObject      *object,
                       guint         property_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
    //APSheet *self = AP_SHEET (object);

    switch (property_id)
    {
        case PROP_FILE:
            //ap_sheet_load_from_file (self, g_value_dup_string (value));
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_sheet_get_property (GObject      *object,
                       guint         property_id,
                       GValue       *value,
                       GParamSpec   *pspec)
{
    //APSheet *self = AP_SHEET (object);
    //APSheetPrivate *priv = ap_sheet_get_instance_private (self);

    switch (property_id)
    {
        case PROP_FILE:
            //g_value_set_string (value, priv->filename);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

/*
 * ap_sheet_finalize:
 * @object: An instance of #APSheet
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
ap_sheet_finalize (GObject *object)
{
   // APSheetPrivate *priv = ap_sheet_get_instance_private (AP_SHEET (object));

    G_OBJECT_CLASS (ap_sheet_parent_class)->finalize (object);
}

/*
 * ap_sheet_dispose:
 * @object: An instance of #APSheet
 *
 * Cleans-up class without full destroying it
 */
static void
ap_sheet_dispose (GObject *object)
{
    //APSheetPrivate *priv = ap_sheet_get_instance_private (AP_SHEET (object));

    G_OBJECT_CLASS (ap_sheet_parent_class)->finalize (object);
}

/*
 * ap_sheet_init:
 * @self: An instance of #APSheet
 *
 * Initializes various aspects of the sheet.
 */
static void
ap_sheet_init (APSheet *self)
{
    APSheetPrivate *priv = ap_sheet_get_instance_private (self);

    priv->characteristics = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
    priv->attributes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
    priv->abilities = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
}

/*
 * ap_sheet_class_init:
 * @klass: An instance of #APSheetClass
 *
 * Initializes the various virtural functions in #APSheetClass.
 */
static void
ap_sheet_class_init (APSheetClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->set_property = ap_sheet_set_property;
    object_class->get_property = ap_sheet_get_property;
    object_class->finalize = ap_sheet_finalize;
    object_class->dispose = ap_sheet_dispose;

    obj_properties[PROP_FILE] =
        g_param_spec_string ("file",
                             "file",
                             "Name of the file to load and display from.",
                             NULL  /* default value */,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       N_PROPERTIES,
                                       obj_properties);
}

/**
 * ap_sheet_new:
 *
 * Creates a new #APSheet, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APSheet *sheet;
 *
 * sheet = ap_sheet_new ();
 * ]|
 *
 * Returns: #APSheet; a sub-object of #GObject
 */
APSheet *
ap_sheet_new (void)
{
    return g_object_new (AP_TYPE_SHEET,
                         NULL);
}

/**
 * ap_sheet_new:
 *
 * Creates a new #APSheet, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * APSheet *sheet;
 *
 * sheet = ap_sheet_from_file ("./my-sheet.xml");
 * ]|
 *
 * Returns: #APSheet; a sub-object of #GObject
 */
APSheet *
ap_sheet_from_file (const gchar *file)
{
    return g_object_new (AP_TYPE_SHEET,
						 "file", file,
                         NULL);
}


